package com.huiju.estateagents.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.StringUtils;
import com.huiju.estateagents.entity.HelpActivity;
import com.huiju.estateagents.entity.HelpInitiateRecord;
import com.huiju.estateagents.entity.HelpRecord;
import com.huiju.estateagents.entity.TaPerson;
import com.huiju.estateagents.mapper.HelpActivityMapper;
import com.huiju.estateagents.mapper.HelpInitiateRecordMapper;
import com.huiju.estateagents.mapper.HelpRecordMapper;
import com.huiju.estateagents.service.IHelpRecordService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 助力活动助力人员记录  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-10-17
 */
@Service
public class HelpRecordServiceImpl extends ServiceImpl<HelpRecordMapper, HelpRecord> implements IHelpRecordService {
	
	@Autowired
	private HelpActivityMapper helpActivityMapper;
	
	@Autowired
	private HelpInitiateRecordMapper helpInitiateRecordMapper;
	
	@Autowired
	private HelpRecordMapper helpRecordMapper;
	
	/**
	 * 助力者助力
	 *
	 * @param helpRecord
	 * @param orgId
	 * @param person
	 * @return
	 */
	@Override
	public ResponseBean saveBooster(HelpRecord helpRecord, Integer orgId, TaPerson person) {
		//查看是否助力成功
		HelpInitiateRecord helpInitiateRecord = helpInitiateRecordMapper.selectById(helpRecord.getHelpRecordInitiateId());
		if (!StringUtils.isEmpty(helpInitiateRecord.getVerification())){
			return ResponseBean.error("已经助力成功", ResponseBean.ERROR_UNAVAILABLE);
		}
		
		HelpActivity helpActivity = helpActivityMapper.selectById(helpRecord.getHelpActivityId());
		//是否助力成功
		if (helpActivity.getEnlistNum() >= helpActivity.getPersonNum()){
			return ResponseBean.error("已经助力成功", ResponseBean.ERROR_UNAVAILABLE);
		}
		if (helpActivity.getEndDate().isBefore(LocalDateTime.now())){
			return ResponseBean.error("活动已超时", ResponseBean.ERROR_UNAVAILABLE);
		}
		//查看每个人的助力次数
		Integer helpNum = helpActivity.getHelpNum();
		
		QueryWrapper<HelpRecord> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("org_id",orgId);
		queryWrapper.eq("help_activity_id",helpRecord.getHelpActivityId());
		queryWrapper.eq("persion_id",person.getPersonId());
		if (null != helpNum && helpNum >= 0){
			//查看这个人是否助力过，和助力了几次
			List<HelpRecord> helpRecords = helpRecordMapper.selectList(queryWrapper);
			if (helpRecords.size() >= helpNum){
				return ResponseBean.error("助力次数达到上限", ResponseBean.ERROR_UNAVAILABLE);
			}
		}
		//查看是否助力过这个人
		queryWrapper.eq("help_record_initiate_id",helpRecord.getHelpRecordInitiateId());
		List<HelpRecord> helpRecordList = helpRecordMapper.selectList(queryWrapper);
		if (helpRecordList.size() > 0){
			return ResponseBean.error("您已经助力过了,无需重复助力", ResponseBean.ERROR_UNAVAILABLE);
		}
		//反更新在入库
		helpActivity.setEnlistNum(helpActivity.getEnlistNum()+1);
		helpActivityMapper.updateById(helpActivity);
		//插入数据
		helpRecord.setOrgId(orgId);
		helpRecord.setCreateDate(LocalDateTime.now());
		helpRecord.setPersonId(person.getPersonId());
		helpRecordMapper.insert(helpRecord);
		if (helpActivity.getEnlistNum().equals(helpActivity.getPersonNum())){
			//发送微信模板消息所有人都发送--todo
			
			//生成核销码
			helpInitiateRecord.setVerification(String.valueOf(helpInitiateRecord.getHelpRecordInitiateId()+System.currentTimeMillis()));
			helpInitiateRecord.setSucceedDate(LocalDateTime.now());
			helpInitiateRecord.setStatus(0);
		}
		helpInitiateRecord.setEnlistNum(helpInitiateRecord.getEnlistNum()+1);
		helpInitiateRecordMapper.updateById(helpInitiateRecord);
		return ResponseBean.success(helpInitiateRecord);
	}

}
